var path = require("path");
const HtmlWebpackPlugin = require("html-webpack-plugin");
const VueLoaderPlugin = require("vue-loader/lib/plugin");

var config = {
  entry: "./src/main.js",
  output: {
    path: path.resolve(__dirname + "/dist"),
    filename: "[name].build.js"
  },
  module: {
    rules: [
      {
        test: /\.vue$/,
        loader: "vue-loader"
      },
      {
        test: /\.js$/,
        loader: "babel-loader",
        exclude: /node_modules/
      },
      {
        test: /\.tsx?$/,
        loader: "ts-loader",
        exclude: /node_modules/,
        options: {
          appendTsSuffixTo: [/\.vue$/]
        }
      },
      {
        test: /\.css$/,
        use: ["vue-style-loader", "css-loader"]
      },
      {
        test: /\.scss$/,
        use: ["vue-style-loader", "css-loader", "sass-loader"]
      },
      {
        test: /\.(jpg|png)$/,
        use: {
          loader: "url-loader",
          options: {
            limit: 25000
          }
        }
      }
    ]
  },
  resolve: {
    // 需要打包的文件后缀
    extensions: [".tsx", ".ts", ".js", ".vue"]
  },
  plugins: [
    new HtmlWebpackPlugin({
      template: "public/index.html"
    }),
    new VueLoaderPlugin()
  ],
  devServer: {
    contentBase: path.join(__dirname, "public"),
    compress: true, // 是否压缩
    disableHostCheck: true,
    host: "0.0.0.0",
    https: false,
    port: 9000, // 启动服务端口
    hot: true, // 是否自动刷新
    open: true // 是否启动服务后，自动打开浏览器
  }
};
module.exports = config;
