"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Observable = void 0;
var Observable = /** @class */ (function () {
    function Observable(subscribeAction) {
        this.subscribeAction = subscribeAction;
    }
    Observable.prototype.subscribe = function (oOrOnNext, onError, onCompleted) {
        var observer = new Observer(oOrOnNext, onError, onCompleted);
        var result = this.subscribeAction(observer); // 开始发射数据并拿到subscribeAction的返回值
        return new Subscription(observer, result); // 创造控制observer和subscribeAction的实例对象
    };
    return Observable;
}());
exports.Observable = Observable;
var Observer = /** @class */ (function () {
    function Observer(onNext, onError, onCompleted) {
        this.isStopped = false;
        if (typeof onNext === "object") {
            this._onNext = onNext.next;
            this._onError = onNext.error;
            this._onCompleted = onNext.complete;
        }
        else {
            this._onNext = onNext;
            this._onError = onError;
            this._onCompleted = onCompleted;
        }
    }
    Observer.prototype.next = function (value) {
        if (!this.isStopped && this._onNext) {
            this._onNext(value);
        }
    };
    Observer.prototype.error = function (err) {
        if (!this.isStopped && this._onError) {
            this.isStopped = true;
            this._onError(err);
        }
    };
    Observer.prototype.complete = function (res) {
        if (!this.isStopped && this._onCompleted) {
            this.isStopped = true;
            this._onCompleted(res);
        }
    };
    return Observer;
}());
var Subscription = /** @class */ (function () {
    function Subscription(observer, result) {
        this.observer = observer;
        this.result = result;
    }
    Subscription.prototype.unsubscribe = function () {
        this.observer.isStopped = true; // 取消observer的订阅
        this.result();
    };
    return Subscription;
}());
//# sourceMappingURL=observable.js.map