"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Pool = void 0;
var ERR_CODE_NETWORK_TYPE = 10005;
var utils_1 = require("./utils");
var Pool = /** @class */ (function () {
    function Pool(runTask, limit) {
        this.runTask = runTask;
        this.queue = [];
        this.processing = [];
        this.limit = limit;
    }
    Pool.prototype.enqueue = function (task) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.queue.push({
                task: task,
                resolve: resolve,
                reject: reject
            });
            _this.check();
        });
    };
    Pool.prototype.run = function (item) {
        var _this = this;
        this.queue = this.queue.filter(function (v) { return v !== item; });
        this.processing.push(item);
        this.runTask(item.task).then(function (res) {
            _this.processing = _this.processing.filter(function (v) { return v !== item; });
            var eventvalue = JSON.stringify(res || {});
            utils_1.sendTracking({
                eventid: 1099562,
                eventtype: "upload-chunk-res",
                eventvalue: eventvalue
            });
            if (res) {
                if (res === "1") {
                    item.resolve(res);
                }
                if (res.data) {
                    if (res.data.code) {
                        item.reject({
                            code: ERR_CODE_NETWORK_TYPE,
                            msg: res.data.msg
                        });
                    }
                    else {
                        if (res.data.status === 200) {
                            item.resolve(res.data.data);
                        }
                        else {
                            item.reject({
                                code: ERR_CODE_NETWORK_TYPE,
                                msg: res.data.msg
                            });
                        }
                    }
                }
                else {
                    item.reject({
                        code: ERR_CODE_NETWORK_TYPE,
                        msg: "网络错误"
                    });
                }
            }
            else {
                item.reject({
                    code: ERR_CODE_NETWORK_TYPE,
                    msg: "网络错误"
                });
            }
            _this.check();
        }, function (err) { return item.reject(err); });
    };
    Pool.prototype.check = function () {
        var _this = this;
        var processingNum = this.processing.length;
        var availableNum = this.limit - processingNum;
        this.queue.slice(0, availableNum).forEach(function (item, index) {
            _this.run(item);
        });
    };
    return Pool;
}());
exports.Pool = Pool;
//# sourceMappingURL=pool.js.map