"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UploadManager = void 0;
var utils_1 = require("./utils");
var progressbeforeupload_1 = require("./progressbeforeupload");
var pool_1 = require("./pool");
var env_1 = require("./env");
// 文件分块大小
// let BLOCK_SIZE = 2m * 1024 * 1024;
// 限制文件上传类型
var ERR_CODE_FILE_TYPE = 10001;
// 视频
var ERR_CODE_FILE_VIDEO_SIZE = 10002;
// 图片
var ERR_CODE_FILE_IMAGE_SIZE = 10003;
// 上传失败
var ERR_CODE_SEND_TYPE = 10004;
// 图片格式
var imageType = ["image/png", "image/jpeg", "image/jpg"];
// 视频格式
var videoType = ["video/mp4"];
// 文本格式
var txtType = [
    "text/plain",
    "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
    "application/vnd.ms-excel",
    "text/csv",
    "text/xml",
    "application/xml"
];
//
var REQUEST_SOURCE_EXCLUDES = ["fanstop", "kabp", "superfansfile"];
//upload_id过期
var ERR_CODE_INVALID_UPLOADID = 10005;
//缺少customerid 或 source 参数
var ERR_CODE_QUYER = 10006;
var ERR_CODE_QUYER_MSG = "缺少参数";
var host = env_1.default.get('host');
var intranet_host = env_1.default.get('intranet_host');
var MAX = 512;
var MAX_IMG = 5;
var imageUploadApi = "/api/v1/image/upload";
var imageUploadApiWep = "/api/v1/image/uploadWebp";
var UploadManager = /** @class */ (function () {
    function UploadManager(options, handlers, statisticsLogger) {
        var _this = this;
        this.config = Object.assign({
            retryCount: 3,
            checkByMD5: false,
            forceDirect: false,
            concurrentRequestLimit: 3
        }, options.config);
        this.extension = Object.assign({
            fname: "",
            params: {},
            mimeType: null,
            downLoadMimeType: null,
            blockSize: 10 * 1024 * 1024
        }, options.extension);
        this.statisticsLogger = statisticsLogger;
        this.progress = null;
        this.xhrList = [];
        this.xhrHandler = function (xhr) { return _this.xhrList.push(xhr); };
        this.aborted = false;
        this.file = options.file;
        this.key = options.key;
        this.onData = function () { };
        this.onError = function () { };
        this.onComplete = function () { };
        this.retryCount = 0;
        Object.assign(this, handlers);
    }
    UploadManager.prototype.putFile = function () {
        var _this = this;
        this.aborted = false;
        if (!isNaN(this.extension.maxSize)) {
            MAX = this.extension.maxSize;
        }
        if (!isNaN(this.extension.maxSizeImg)) {
            MAX_IMG = this.extension.maxSizeImg;
        }
        // 文件名称
        if (!this.extension.fname) {
            this.extension.fname = this.file.name;
        }
        // 文件类型
        if (this.extension.mimeType && this.extension.mimeType.length) {
            if (!utils_1.isContainFileMimeType(this.file.type, this.extension.mimeType)) {
                this.onError({
                    code: ERR_CODE_FILE_TYPE,
                    msg: "文件类型错误"
                });
                return;
            }
        }
        // 文件大小 图片不超过5m 视频不超过500m
        if (/image/.test(this.file.type)) {
            if (this.file.size > MAX_IMG * 1024 * 1024) {
                this.onError({
                    code: ERR_CODE_FILE_IMAGE_SIZE,
                    msg: "\u56FE\u7247\u5927\u5C0F\u4E0D\u80FD\u8D85\u8FC7" + MAX_IMG + "M"
                });
            }
        }
        else if (/video/.test(this.file.type)) {
            if (this.file.size > MAX * 1024 * 1024) {
                this.onError({
                    code: ERR_CODE_FILE_VIDEO_SIZE,
                    msg: "\u89C6\u9891\u5927\u5C0F\u4E0D\u80FD\u8D85\u8FC7 " + (MAX < 1024 ? MAX + "M" : (MAX / 1024).toFixed(1) + "G")
                });
                return;
            }
        }
        else if (/text/.test(this.file.type)) {
            if (this.file.size > 1024 * 1024 * 1024) {
                this.onError({
                    code: ERR_CODE_FILE_VIDEO_SIZE,
                    msg: "txt文件不能超过1024M"
                });
                return;
            }
        }
        else {
            if (this.file.size > 1024 * 1024 * 1024) {
                this.onError({
                    code: ERR_CODE_FILE_VIDEO_SIZE,
                    msg: "文件大小不能超过1024M"
                });
                return;
            }
        }
        utils_1.sendTracking({
            eventid: 1099562,
            eventtype: "upload-file-before"
        });
        var startTime = new Date().getTime();
        var upload = new Promise(function (resolve, reject) {
            _this.uploadAt = new Date().getTime();
            progressbeforeupload_1.updateProgress("upload");
            if ((_this.extension.uploadMode === "webp" ||
                _this.config.forceDirect ||
                /image/.test(_this.file.type)) &&
                _this.extension.uploadMode !== "file") {
                resolve(_this.directUpload());
            }
            else {
                resolve(_this.resumeUpload());
            }
        });
        upload.then(function (res) {
            var duration = new Date().getTime() - startTime;
            utils_1.sendTracking({
                eventid: 1099562,
                eventtype: "upload-file-success",
                eventvalue: duration
            });
            _this.onComplete(res);
        }, function (err) {
            utils_1.sendTracking({
                eventid: 1099562,
                eventtype: "upload-chunk-err",
                eventvalue: JSON.stringify(err)
            });
            _this.clear();
            if (err.isRequestError && !_this.config.disableStatisticsReport) {
                var reqId = _this.aborted ? "" : err.reqId;
                var code = _this.aborted ? -2 : err.code;
            }
            var needRetry = err.isRequestError && err.code === 0 && !_this.aborted;
            var notReachRetryCount = ++_this.retryCount <= _this.config.retryCount;
            if (needRetry && notReachRetryCount) {
                _this.putFile();
                return;
            }
            _this.onError(err);
        });
        return upload;
    };
    UploadManager.prototype.clear = function () {
        this.xhrList.forEach(function (xhr) { return xhr.abort(); });
        this.xhrList = [];
    };
    UploadManager.prototype.stop = function () {
        this.clear();
        this.aborted = true;
    };
    // 直传
    UploadManager.prototype.directUpload = function () {
        var _this = this;
        var formData = new FormData();
        formData.append("file", this.file);
        if (/image/.test(this.file.type)) {
            formData.append("picName", this.key);
        }
        this.extension.params.request_source &&
            formData.append("request_source", this.extension.params.request_source);
        formData.append("request_time", new Date().getTime());
        formData.append("customer_id", this.extension.params.customerId);
        // let domin = this.extension.params.isIntranet ? intranet_host : host;
        var domain = this.extension.requestDomain
            ? this.extension.requestDomain
            : this.extension.params.isIntranet
                ? intranet_host
                : host;
        if (this.extension.uploadMode === "webp") {
            var requestPath_1 = imageUploadApiWep;
            return utils_1.computeMd5(this.file).then(function (computedMd5) {
                var md5 = _this.extension.params.md5 || computedMd5;
                _this.uploadUrl = "" + domain + requestPath_1 + "?customer_id=" + _this.extension.params.customerId + "&request_source=" + _this.extension.params.request_source + "&key=" + _this.extension.params.key + "&pic_name=" + _this.extension.params.pic_name + "&md5=" + md5;
                if (_this.extension.params.quality) {
                    _this.uploadUrl += "&quality=" + _this.extension.params.quality;
                }
                if (_this.extension.params.lossLess) {
                    _this.uploadUrl += "&lossLess=" + _this.extension.params.lossLess;
                }
                return utils_1.request(_this.uploadUrl, {
                    method: "POST",
                    body: formData,
                    withCredentials: !_this.extension.params.isIntranet,
                    onProgress: function (data) {
                        _this.updateDirectProgress(data.loaded, data.total);
                    },
                    onCreate: _this.xhrHandler
                }).then(function (res) {
                    _this.finishDirectProgress();
                    return res;
                });
            });
        }
        else {
            var requestPath = this.extension.requestPath || imageUploadApi;
            this.uploadUrl = "" + domain + requestPath + "?customer_id=" + this.extension.params.customerId + "&request_source=" + this.extension.params.request_source;
            return utils_1.request(this.uploadUrl, {
                method: "POST",
                body: formData,
                withCredentials: !this.extension.params.isIntranet,
                onProgress: function (data) {
                    _this.updateDirectProgress(data.loaded, data.total);
                },
                onCreate: this.xhrHandler
            }).then(function (res) {
                _this.finishDirectProgress();
                return res;
            });
        }
    };
    // 分片上传
    UploadManager.prototype.resumeUpload = function () {
        var _this = this;
        this.loaded = {
            mkFileProgress: 0,
            chunks: null
        };
        this.ctxList = [];
        this.localInfo = utils_1.getLocalFileInfo(this.file);
        this.chunks = utils_1.getChunks(this.file, this.extension.params.blockSize);
        this.initChunksProgress();
        var pool = new pool_1.Pool(function (chunkInfo) { return _this.uploadChunk(chunkInfo); }, this.config.concurrentRequestLimit);
        var uploadChunks = this.chunks.map(function (chunk, index) {
            return pool.enqueue({
                chunk: chunk,
                index: index
            });
        });
        // 所有任务上传完成
        var result = Promise.all(uploadChunks).then(function (res) {
            return _this.mkFileReq(res);
        });
        var __this = this;
        // 所有任务上传完成
        result.then(function (res) {
            window.localStorage.removeItem(_this.extension.params.md5);
            utils_1.removeLocalFileInfo(_this.file);
        }, function (err) {
            // removeLocalFileInfo(this.file);
            return;
        });
        return result;
    };
    UploadManager.prototype.uploadChunk = function (chunkInfo) {
        var _this = this;
        var index = chunkInfo.index, chunk = chunkInfo.chunk;
        var info = this.localInfo[index];
        var requestUrl;
        if (!this.extension.params.request_source ||
            !this.extension.params.customerId) {
            return Promise.reject({
                code: ERR_CODE_QUYER,
                msg: ERR_CODE_QUYER_MSG
            });
        }
        // let domin = this.extension.params.isIntranet ? intranet_host : host;
        var domain = this.extension.requestDomain
            ? this.extension.requestDomain
            : this.extension.params.isIntranet
                ? intranet_host
                : host;
        if (/video/.test(this.file.type) && this.extension.uploadMode !== "file") {
            var requestPath = this.extension.requestPath || "/api/v1/video/media/upload";
            requestUrl = "" + domain + requestPath + "?customer_id=" + this.extension.params.customerId + "&request_source=" + this.extension.params.request_source;
        }
        else {
            var fileRequestPath = this.extension.requestPath || "/api/v1/file/upload";
            requestUrl = "" + domain + fileRequestPath + "?customer_id=" + this.extension.params.customerId + "&request_source=" + this.extension.params.request_source;
            if (this.extension.params.upload_type) {
                requestUrl += "&upload_type=" + this.extension.params.upload_type;
            }
        }
        // 是否过期
        var savedReusable = info && !utils_1.isChunkExpired(info.time);
        var shouldCheckMD5 = this.config.checkByMD5;
        var reuseSaved = function () {
            _this.updateChunkProgress(chunk.size, index);
            _this.ctxList[index] = {
                ctx: info.ctx,
                size: info.size,
                time: info.time,
                md5: info.md5
            };
            return Promise.resolve("1");
        };
        if (savedReusable && !shouldCheckMD5) {
            return reuseSaved();
        }
        return utils_1.computeMd5(chunk).then(function (md5) {
            // 每个文件分块做md5 跟本地存储的文件块md5比对
            if (savedReusable && md5 === info.md5) {
                return reuseSaved();
            }
            var onProgress = function (data) {
                _this.updateChunkProgress(data.loaded, index);
            };
            var onCreate = _this.xhrHandler;
            var method = "POST";
            var __this = _this;
            var fileSlice = new File([chunk], md5, {
                type: _this.file.type
            });
            var formData = new FormData();
            if (!/video/.test(_this.file.type) ||
                _this.extension.uploadMode === "file") {
                //文件上传参数
                formData.append("name", __this.key);
                formData.append("file_length", __this.file.size);
                formData.append("check", __this.extension.params.md5.split("_")[0]);
                formData.append("file", fileSlice);
                formData.append("slice_check", md5);
                formData.append("part_number", index);
                formData.append("length", chunk.size);
                formData.append("upload_id", __this.extension.params.uploadId);
                formData.append("mimetype", __this.extension.downLoadMimeType);
                if (!REQUEST_SOURCE_EXCLUDES.includes(__this.extension.params.request_source)) {
                    formData.append("request_source", __this.extension.params.request_source);
                    formData.append("customer_id", __this.extension.params.customerId);
                }
                __this.extension.params.file_url &&
                    formData.append("file_url", __this.extension.params.file_url);
            }
            else {
                //视频上传参数
                formData.append("file", fileSlice);
                formData.append("length", __this.extension.params.blockSize);
                formData.append("check", __this.extension.params.md5);
                formData.append("upload_id", __this.extension.params.uploadId);
                formData.append("part_number", index);
                formData.append("slice_check", md5);
                formData.append("file_length", chunk.size);
                formData.append("request_source", __this.extension.params.request_source);
                formData.append("request_time", new Date().getTime());
                formData.append("customer_id", __this.extension.params.customerId);
            }
            utils_1.sendTracking({
                eventid: 1099562,
                eventtype: "before-upload-chunk" // 事件类型 ，非必须
            });
            var startTime = new Date().getTime();
            return utils_1.request(requestUrl, {
                method: method,
                body: formData,
                withCredentials: !__this.extension.params.isIntranet,
                onProgress: onProgress,
                onCreate: onCreate
            })
                .then(function (response) {
                onProgress({
                    loaded: chunk.size
                });
                _this.ctxList[index] = {
                    time: new Date().getTime(),
                    ctx: response.data.ctx,
                    size: chunk.size,
                    md5: md5
                };
                var eventvalue = __assign({ duration: new Date().getTime() - startTime, part_number: index, file_length: chunk.size, customer_id: __this.extension.params.customerId, upload_id: __this.extension.params.uploadId, request_source: __this.extension.params.request_source, md5: __this.extension.params.md5.split("_")[0] ||
                        __this.extension.params.md5 }, response.data);
                utils_1.sendTracking({
                    eventid: 1099562,
                    eventtype: "upload-chunk",
                    eventvalue: JSON.stringify(eventvalue)
                });
                if (response.data && response.data.data) {
                    utils_1.setLocalFileInfo(_this.file, _this.ctxList);
                }
                return response;
            })
                .catch(function (err) {
                var eventvalue = {
                    part_number: index,
                    file_length: chunk.size,
                    customer_id: __this.extension.params.customerId,
                    upload_id: __this.extension.params.uploadId,
                    request_source: __this.extension.params.request_source,
                    sliceMd5: md5,
                    md5: __this.extension.params.md5.split("_")[0] ||
                        __this.extension.params.md5,
                    error: err
                };
                utils_1.sendTracking({
                    eventid: 1099562,
                    eventtype: "upload-chunk-request-err",
                    eventvalue: JSON.stringify(eventvalue)
                });
            });
        });
    };
    UploadManager.prototype.mkFileReq = function (data) {
        var _this = this;
        var fid;
        if (!/video/.test(this.file.type)) {
            data.map(function (item) {
                if (item.downLoadUrl && item.downLoadUrl !== "") {
                    fid = item.downLoadUrl;
                }
            });
        }
        else {
            data.map(function (item) {
                if (item.fid) {
                    fid = item.fid;
                }
            });
        }
        return new Promise(function (resolve, reject) {
            _this.updateMkFileProgress(1);
            resolve(fid);
        });
    };
    UploadManager.prototype.updateDirectProgress = function (loaded, total) {
        // 当请求未完成时可能进度会达到100，所以total + 1来防止这种情况出现
        this.progress = {
            total: this.getProgressInfoItem(loaded, total + 1)
        };
        this.onData(this.progress);
    };
    UploadManager.prototype.finishDirectProgress = function () {
        // 在某些浏览器环境下，xhr 的 progress 事件无法被触发，progress 为 null，
        if (!this.progress) {
            this.progress = {
                total: this.getProgressInfoItem(this.file.size, this.file.size)
            };
            this.onData(this.progress);
            return;
        }
        var total = this.progress.total;
        this.progress = {
            total: this.getProgressInfoItem(total.loaded + 1, total.size)
        };
        this.onData(this.progress);
    };
    UploadManager.prototype.initChunksProgress = function () {
        this.loaded.chunks = this.chunks.map(function (_) { return 0; });
        this.notifyResumeProgress();
    };
    UploadManager.prototype.updateChunkProgress = function (loaded, index) {
        this.loaded.chunks[index] = loaded;
        this.notifyResumeProgress();
    };
    UploadManager.prototype.updateMkFileProgress = function (progress) {
        this.loaded.mkFileProgress = progress;
        this.notifyResumeProgress();
    };
    UploadManager.prototype.notifyResumeProgress = function () {
        var _this = this;
        this.progress = {
            total: this.getProgressInfoItem(utils_1.sum(this.loaded.chunks) + this.loaded.mkFileProgress, this.file.size + 1),
            chunks: this.chunks.map(function (chunk, index) {
                return _this.getProgressInfoItem(_this.loaded.chunks[index], chunk.size);
            })
        };
        this.onData(this.progress);
    };
    UploadManager.prototype.getProgressInfoItem = function (loaded, size) {
        return {
            loaded: loaded,
            size: size,
            percent: (loaded / size) * 100
        };
    };
    return UploadManager;
}());
exports.UploadManager = UploadManager;
//# sourceMappingURL=upload.js.map