"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.file2ArrayBuffer = exports.isContainFileMimeType = exports.request = exports.computeMd5 = exports.readAsArrayBuffer = exports.computeMd5Incrementally = exports.createXHR = exports.getResumeUploadedSize = exports.getLocalFileInfo = exports.removeLocalFileInfo = exports.setLocalFileInfo = exports.sum = exports.getChunks = exports.isChunkExpired = exports.sendTracking = exports.send = exports.getCheka = void 0;
var SparkMD5 = require("spark-md5");
var progressbeforeupload_1 = require("./progressbeforeupload");
function getCheka() {
    if (window.wbadmt && window.wbadmt.send) {
        return window.wbadmt;
    }
    return null;
}
exports.getCheka = getCheka;
function send(data, type, isImmediately) {
    if (type === void 0) { type = null; }
    if (isImmediately === void 0) { isImmediately = false; }
    var c = getCheka();
    if (c) {
        c.send(data, type, isImmediately);
        return true;
    }
    return false;
}
exports.send = send;
function sendTracking(data) {
    if (!send(data)) {
        window.addEventListener("cheka:ready", function () {
            send(data);
        }, false);
    }
}
exports.sendTracking = sendTracking;
function isChunkExpired(time) {
    var expireAt = time + 3600 * 24 * 1000;
    return new Date().getTime() > expireAt;
}
exports.isChunkExpired = isChunkExpired;
// 文件分块
function getChunks(file, blockSize) {
    var chunks = [];
    var count = Math.ceil(file.size / blockSize);
    for (var i = 0; i < count; i++) {
        var chunk = file.slice(blockSize * i, i === count - 1 ? file.size : blockSize * (i + 1));
        chunks.push(chunk);
    }
    return chunks;
}
exports.getChunks = getChunks;
function sum(list) {
    return list.reduce(function (sum, loaded) {
        return sum + loaded;
    }, 0);
}
exports.sum = sum;
function setLocalFileInfo(file, info) {
    try {
        localStorage.setItem(createLocalKey(file), JSON.stringify(info));
    }
    catch (err) {
        if (window.console && window.console.warn) {
            console.warn("setLocalFileInfo failed");
        }
    }
}
exports.setLocalFileInfo = setLocalFileInfo;
function createLocalKey(file) {
    return "wb_ad_js_sdk_upload_" + file.name + "_size_" + file.size;
}
function removeLocalFileInfo(file) {
    try {
        localStorage.removeItem(createLocalKey(file));
    }
    catch (err) {
        if (window.console && window.console.warn) {
            console.warn("removeLocalFileInfo failed");
        }
    }
}
exports.removeLocalFileInfo = removeLocalFileInfo;
function getLocalFileInfo(file) {
    try {
        return JSON.parse(localStorage.getItem(createLocalKey(file))) || [];
    }
    catch (err) {
        if (window.console && window.console.warn) {
            console.warn("getLocalFileInfo failed");
        }
        return [];
    }
}
exports.getLocalFileInfo = getLocalFileInfo;
function getResumeUploadedSize(file) {
    return getLocalFileInfo(file)
        .filter(function (value) { return value && !isChunkExpired(value.time); })
        .reduce(function (result, value) { return result + value.size; }, 0);
}
exports.getResumeUploadedSize = getResumeUploadedSize;
function getAuthHeaders(token) {
    var auth = "UpToken " + token;
    return {
        Authorization: auth
    };
}
function createXHR() {
    if (window.XMLHttpRequest) {
        return new XMLHttpRequest();
    }
    return new window.ActiveXObject("Microsoft.XMLHTTP");
}
exports.createXHR = createXHR;
/**
 *
 * @param {File} data
 * @returns string
 */
function computeMd5Incrementally(data) {
    var blobSlice = File.prototype.slice ||
        File.prototype.mozSlice ||
        File.prototype.webkitSlice, chunkSize = 2097152 * 5, // Read in chunks of 10MB
    chunks = Math.ceil(data.size / chunkSize), currentChunk = 0, spark = new SparkMD5.ArrayBuffer(), fileReader = new FileReader();
    function loadNext() {
        var start = currentChunk * chunkSize, end = start + chunkSize >= data.size ? data.size : start + chunkSize;
        fileReader.readAsArrayBuffer(blobSlice.call(data, start, end));
    }
    return new Promise(function (resolve, reject) {
        fileReader.onload = function (e) {
            console.log("分段计算 md5，已完成第", currentChunk + 1, "段，总计", chunks, "段");
            progressbeforeupload_1.updateProgressCount(Math.floor(((currentChunk + 1) / chunks) * 100));
            spark.append(e.target.result); // Append array buffer
            currentChunk++;
            if (currentChunk < chunks) {
                loadNext();
            }
            else {
                var md5 = spark.end();
                console.log("md5 计算完成：", md5);
                resolve(md5);
            }
        };
        fileReader.onerror = function (error) {
            console.warn(error);
            reject({ msg: "计算md5值失败", error: error });
        };
        loadNext();
    });
}
exports.computeMd5Incrementally = computeMd5Incrementally;
function readAsArrayBuffer(data) {
    return new Promise(function (resolve, reject) {
        var reader = new FileReader();
        reader.readAsArrayBuffer(data);
        reader.onload = function (evt) {
            var data = evt.target.result;
            resolve(data);
        };
        reader.onerror = function () {
            reject(new Error("读取错误"));
        };
    });
}
exports.readAsArrayBuffer = readAsArrayBuffer;
function computeMd5(data) {
    return readAsArrayBuffer(data).then(function (buffer) {
        var spark = new SparkMD5.ArrayBuffer();
        spark.append(buffer);
        return spark.end();
    });
}
exports.computeMd5 = computeMd5;
function request(url, options) {
    return new Promise(function (resolve, reject) {
        var xhr = createXHR();
        xhr.open(options.method, url);
        if (options.onCreate) {
            options.onCreate(xhr);
        }
        xhr.withCredentials = options.hasOwnProperty("withCredentials")
            ? options.withCredentials
            : true;
        xhr.upload.addEventListener("progress", function (evt) {
            if (evt.lengthComputable && options.onProgress) {
                options.onProgress({
                    loaded: evt.loaded,
                    total: evt.total
                });
            }
        });
        xhr.addEventListener("timeout", function (evt) {
            sendTracking({
                eventid: 1099562,
                eventtype: "xhr-timeout",
                eventvalue: evt
            });
        });
        xhr.addEventListener("error", function (evt) {
            sendTracking({
                eventid: 1099562,
                eventtype: "xhr-error",
                eventvalue: evt
            });
        });
        xhr.addEventListener("abort", function (evt) {
            sendTracking({
                eventid: 1099562,
                eventtype: "xhr-abort",
                eventvalue: evt
            });
        });
        xhr.onreadystatechange = function () {
            var responseText = xhr.responseText;
            if (xhr.readyState !== 4) {
                return;
            }
            if (xhr.status !== 200) {
                var message = "xhr request failed, code: " + xhr.status + ";";
                if (responseText) {
                    message = message + (" response: " + responseText);
                }
                reject({
                    code: xhr.status,
                    message: message,
                    isRequestError: true
                });
                return;
            }
            try {
                resolve({
                    data: JSON.parse(responseText)
                });
            }
            catch (err) {
                reject(err);
            }
        };
        xhr.send(options.body);
    });
}
exports.request = request;
function isContainFileMimeType(fileType, mimeType) {
    return mimeType.indexOf(fileType) > -1;
}
exports.isContainFileMimeType = isContainFileMimeType;
function file2ArrayBuffer(file) {
    var reader = new FileReader();
    reader.readAsArrayBuffer(file);
    return new Promise(function (resolve) {
        reader.onload = function () {
            var content = reader.result;
            resolve(content);
        };
    }).then(function (arrayBuffer) {
        return arrayBuffer;
    });
}
exports.file2ArrayBuffer = file2ArrayBuffer;
//# sourceMappingURL=utils.js.map