const shell = require("shelljs");
const { version } = require("./package.json");
const fs = require("fs");
function incrementVersion(version) {
  const versionParts = version.split(".");
  const lastPart = versionParts.pop();
  versionParts.push(parseInt(lastPart) + 1);
  return versionParts.join(".");
}

// 检查当前git工作区是否clean
const gitStatus = shell.exec("git status --porcelain").stdout.trim();
if (gitStatus) {
  console.log("当前git工作区不是clean状态，请先提交或清理变更");
  process.exit(1);
}

// 检查当前所处git分支是否是master
const branch = shell.exec("git rev-parse --abbrev-ref HEAD").stdout.trim();
if (branch !== "master") {
  console.log("当前git分支不是master，仅能发布beta版本");
  // 自增版本号的最后一位
  let newVersion = version + "beta";
  // 运行'npm run build'命令进行编译
  if (shell.exec("npm run build").code !== 0) {
    console.log("编译失败");
    process.exit(1);
  }
  // 写入新的版本号到package.json
  const packageJsonPath = "./package.json";
  const packageJson = JSON.parse(fs.readFileSync(packageJsonPath, "utf8"));
  packageJson.version = newVersion;
  fs.writeFileSync(packageJsonPath, JSON.stringify(packageJson, null, 2));
  if (
    shell.exec(
      "npm publish --access public --registry=http://nexus.biz.weibo.com/repository/npm-hosts --_authToken=NpmToken.5f307cd5-3e3b-3757-b949-4fe70f643868"
    ).code !== 0
  ) {
    console.log("发布失败");
    process.exit(1);
  }
  packageJson.version = version;
  fs.writeFileSync(packageJsonPath, JSON.stringify(packageJson, null, 2));
} else {
  // 自增版本号的最后一位
  const newVersion = incrementVersion(version);

  // 运行'npm run build'命令进行编译
  if (shell.exec("npm run build").code !== 0) {
    console.log("编译失败");
    process.exit(1);
  }

  // 写入新的版本号到package.json
  const packageJsonPath = "./package.json";
  const packageJson = JSON.parse(fs.readFileSync(packageJsonPath, "utf8"));
  packageJson.version = newVersion;
  fs.writeFileSync(packageJsonPath, JSON.stringify(packageJson, null, 2));

  if (
    shell.exec(
      "npm publish --access public --registry=http://nexus.biz.weibo.com/repository/npm-hosts --_authToken=NpmToken.6aed556c-686c-3bf0-941c-3fd5637744cf"
    ).code !== 0
  ) {
    console.log("发布失败");
    process.exit(1);
  }

  // 创建新的git commit
  shell.exec("git add .");
  shell.exec(`git commit -m "Release version ${newVersion}"`);

  console.log(`成功发布新版本 ${newVersion}`);
}
