const ERR_CODE_NETWORK_TYPE = 10005;
import { sendTracking } from "./utils";

export class Pool {
  constructor(runTask, limit) {
    this.runTask = runTask;
    this.queue = [];
    this.processing = [];
    this.limit = limit;
  }

  enqueue(task) {
    return new Promise((resolve, reject) => {
      this.queue.push({
        task,
        resolve,
        reject
      });
      this.check();
    });
  }
  run(item) {
    this.queue = this.queue.filter(v => v !== item);
    this.processing.push(item);
    this.runTask(item.task).then(
      res => {
        this.processing = this.processing.filter(v => v !== item);
        let eventvalue = JSON.stringify(res || {});
        sendTracking({
          eventid: 1099562, // 申请的事件id
          eventtype: "upload-chunk-res", // 事件类型 ，非必须
          eventvalue
        });
        if (res) {
          if (res === "1") {
            item.resolve(res);
          }
          if (res.data) {
            if (res.data.code) {
              item.reject({
                code: ERR_CODE_NETWORK_TYPE,
                msg: res.data.msg
              });
            } else {
              if (res.data.status === 200) {
                item.resolve(res.data.data);
              } else {
                item.reject({
                  code: ERR_CODE_NETWORK_TYPE,
                  msg: res.data.msg
                });
              }
            }
          } else {
            item.reject({
              code: ERR_CODE_NETWORK_TYPE,
              msg: "网络错误"
            });
          }
        } else {
          item.reject({
            code: ERR_CODE_NETWORK_TYPE,
            msg: "网络错误"
          });
        }
        this.check();
      },
      err => item.reject(err)
    );
  }
  check() {
    let processingNum = this.processing.length;
    let availableNum = this.limit - processingNum;
    this.queue.slice(0, availableNum).forEach((item, index) => {
      this.run(item);
    });
  }
}
