const PROGRESS_EVENT_NAME = "progress_before_upload";

// https://yemdpjdih2.feishu.cn/docx/ApSidkjqxoFEKLxkQdIc3bfTneg
const UNIT_TIME = 0.012 * 1000;

export const progress = {
  mocknumber: 0
};

window.progress = {
  mocknumber: progress.mocknumber
};

let timer = null;

export const PROGRESS_MAP = {
  auto: 0,
  fileread: 20,
  md5: 50,
  storage: 80,
  upload: 100
};

export function observerProgress(progressCallback) {
  progress.mocknumber = 0;
  window.addEventListener(PROGRESS_EVENT_NAME, e => {
    const { timeCount } = e.detail;
    progress.mocknumber = Math.max(progress.mocknumber, timeCount);
    progressCallback && progressCallback(progress.mocknumber);
    if (progress.mocknumber >= 100) {
      clearInterval(timer);
      timer = null;
    }
  });
}

export function updateProgressCount(count) {
  var myEvent = new CustomEvent(PROGRESS_EVENT_NAME, {
    detail: {
      timeCount: count
    }
  });
  window.dispatchEvent(myEvent);
}

export function updateProgress(type) {
  let timeCount = 0;
  if (type === "auto") {
    if (progress.mocknumber === 99) {
      timeCount = 99;
    } else {
      timeCount = progress.mocknumber + 1;
    }
  } else {
    timeCount = PROGRESS_MAP[type];
    if (type === "storage") {
      timeCount = Math.max(progress.mocknumber, PROGRESS_MAP[type]);
      clearInterval(timer);
      timer = null;
    }
  }
  updateProgressCount(timeCount);
}

export function setAutoUpdate(size) {
  if (timer) {
    return;
  }
  timer = setInterval(() => {
    queueMicrotask(() => {
      console.log("setAutoUpdate setInterval");
      if (progress.mocknumber >= 100) {
        clearInterval(timer);
        timer = null;
        return;
      }
      updateProgress("auto");
    });
  }, (UNIT_TIME * size) / 100);
}
