import { urlSafeBase64Encode, urlSafeBase64Decode } from "./base64";
import * as SparkMD5 from "spark-md5";
import { updateProgressCount } from "./progressbeforeupload";
export function getCheka() {
  if (window.wbadmt && window.wbadmt.send) {
    return window.wbadmt;
  }
  return null;
}
export function send(data, type = null, isImmediately = false) {
  let c = getCheka();
  if (c) {
    c.send(data, type, isImmediately);
    return true;
  }
  return false;
}

export function sendTracking(data) {
  if (!send(data)) {
    window.addEventListener(
      "cheka:ready",
      function() {
        send(data);
      },
      false
    );
  }
}

export function isChunkExpired(time) {
  let expireAt = time + 3600 * 24 * 1000;
  return new Date().getTime() > expireAt;
}
// 文件分块
export function getChunks(file, blockSize) {
  let chunks = [];
  let count = Math.ceil(file.size / blockSize);
  for (let i = 0; i < count; i++) {
    let chunk = file.slice(
      blockSize * i,
      i === count - 1 ? file.size : blockSize * (i + 1)
    );
    chunks.push(chunk);
  }
  return chunks;
}

export function sum(list) {
  return list.reduce((sum, loaded) => {
    return sum + loaded;
  }, 0);
}

export function setLocalFileInfo(file, info) {
  try {
    localStorage.setItem(createLocalKey(file), JSON.stringify(info));
  } catch (err) {
    if (window.console && window.console.warn) {
      console.warn("setLocalFileInfo failed");
    }
  }
}

function createLocalKey(file) {
  return "wb_ad_js_sdk_upload_" + file.name + "_size_" + file.size;
}

export function removeLocalFileInfo(file) {
  try {
    localStorage.removeItem(createLocalKey(file));
  } catch (err) {
    if (window.console && window.console.warn) {
      console.warn("removeLocalFileInfo failed");
    }
  }
}

export function getLocalFileInfo(file) {
  try {
    return JSON.parse(localStorage.getItem(createLocalKey(file))) || [];
  } catch (err) {
    if (window.console && window.console.warn) {
      console.warn("getLocalFileInfo failed");
    }
    return [];
  }
}

export function getResumeUploadedSize(file) {
  return getLocalFileInfo(file)
    .filter(value => value && !isChunkExpired(value.time))
    .reduce((result, value) => result + value.size, 0);
}

function getAuthHeaders(token) {
  let auth = "UpToken " + token;
  return {
    Authorization: auth
  };
}

export function createXHR() {
  if (window.XMLHttpRequest) {
    return new XMLHttpRequest();
  }
  return new window.ActiveXObject("Microsoft.XMLHTTP");
}

/**
 *
 * @param {File} data
 * @returns string
 */
export function computeMd5Incrementally(data) {
  var blobSlice =
      File.prototype.slice ||
      File.prototype.mozSlice ||
      File.prototype.webkitSlice,
    chunkSize = 2097152 * 5, // Read in chunks of 10MB
    chunks = Math.ceil(data.size / chunkSize),
    currentChunk = 0,
    spark = new SparkMD5.ArrayBuffer(),
    fileReader = new FileReader();
  function loadNext() {
    var start = currentChunk * chunkSize,
      end = start + chunkSize >= data.size ? data.size : start + chunkSize;
    fileReader.readAsArrayBuffer(blobSlice.call(data, start, end));
  }
  return new Promise((resolve, reject) => {
    fileReader.onload = function(e) {
      console.log(
        "分段计算 md5，已完成第",
        currentChunk + 1,
        "段，总计",
        chunks,
        "段"
      );
      updateProgressCount(Math.floor(((currentChunk + 1) / chunks) * 100));
      spark.append(e.target.result); // Append array buffer
      currentChunk++;

      if (currentChunk < chunks) {
        loadNext();
      } else {
        const md5 = spark.end();
        console.log("md5 计算完成：", md5);
        resolve(md5);
      }
    };
    fileReader.onerror = function(error) {
      console.warn(error);
      reject({ msg: "计算md5值失败", error });
    };
    loadNext();
  });
}

export function readAsArrayBuffer(data) {
  return new Promise((resolve, reject) => {
    let reader = new FileReader();
    reader.readAsArrayBuffer(data);
    reader.onload = evt => {
      let data = evt.target.result;
      resolve(data);
    };
    reader.onerror = () => {
      reject(new Error("读取错误"));
    };
  });
}

export function computeMd5(data) {
  return readAsArrayBuffer(data).then(buffer => {
    let spark = new SparkMD5.ArrayBuffer();
    spark.append(buffer);
    return spark.end();
  });
}

export function request(url, options) {
  return new Promise((resolve, reject) => {
    let xhr = createXHR();
    xhr.open(options.method, url);
    if (options.onCreate) {
      options.onCreate(xhr);
    }
    xhr.withCredentials = options.hasOwnProperty("withCredentials")
      ? options.withCredentials
      : true;
    xhr.upload.addEventListener("progress", evt => {
      if (evt.lengthComputable && options.onProgress) {
        options.onProgress({
          loaded: evt.loaded,
          total: evt.total
        });
      }
    });
    xhr.addEventListener("timeout", evt => {
      sendTracking({
        eventid: 1099562, // 申请的事件id
        eventtype: "xhr-timeout", // 事件类型 ，非必须
        eventvalue: evt
      });
    });
    xhr.addEventListener("error", evt => {
      sendTracking({
        eventid: 1099562, // 申请的事件id
        eventtype: "xhr-error", // 事件类型 ，非必须
        eventvalue: evt
      });
    });
    xhr.addEventListener("abort", evt => {
      sendTracking({
        eventid: 1099562, // 申请的事件id
        eventtype: "xhr-abort", // 事件类型 ，非必须
        eventvalue: evt
      });
    });
    xhr.onreadystatechange = () => {
      let responseText = xhr.responseText;
      if (xhr.readyState !== 4) {
        return;
      }
      if (xhr.status !== 200) {
        let message = `xhr request failed, code: ${xhr.status};`;
        if (responseText) {
          message = message + ` response: ${responseText}`;
        }
        reject({
          code: xhr.status,
          message: message,
          isRequestError: true
        });
        return;
      }
      try {
        resolve({
          data: JSON.parse(responseText)
        });
      } catch (err) {
        reject(err);
      }
    };
    xhr.send(options.body);
  });
}

export function isContainFileMimeType(fileType, mimeType) {
  return mimeType.indexOf(fileType) > -1;
}

export function file2ArrayBuffer(file) {
  let reader = new FileReader();
  reader.readAsArrayBuffer(file);
  return new Promise(resolve => {
    reader.onload = function() {
      const content = reader.result;
      resolve(content);
    };
  }).then(arrayBuffer => {
    return arrayBuffer;
  });
}
