import * as BaseUpload from "./base/index.js";
import env from './base/env/index.js'
import {
  computeMd5,
  computeMd5Incrementally,
  file2ArrayBuffer,
  sendTracking
} from "./base/utils";
import {
  observerProgress,
  setAutoUpdate,
  updateProgress
} from "./base/progressbeforeupload.js";
import {
  isValidPicID,
  getImgURL,
  getImageFormat,
  getImgGeometry,
  getImgLargeFileSize
} from "./base/weiboimgbed";
import axios from "axios";
var MP4Box = require("mp4box");
export const FileAxios = axios.create();

interface SliceChunkParams {
  file: File;
  customerId: string;
  request_source: string;
  url?: string;
  isIntranet?: boolean;
  withCredentials?: boolean;
  video_type?: string;
  requestDomain?: string;
  requestPath?: string;
  name?: string;
  file_url?: string;
  progressCallback?: (progress: number) => void;
}

interface IChunkLimit {
  name: string;
  length: number;
  check: string;
  request_time: string;
  source: string;
  customer_id: string;
}
let sendParams: IChunkLimit;

interface UpstreamParam {
  // uid: string;
  source: any;
  assetsHost?: string;
}
// 图片pid
interface Iimage {
  pid: string;
}


const hostAssets = "https://assets.biz.weibo.com";;
const host = env.get('host');
const intranet_host = env.get('intranet_host');

let checkTimehost = "";
if (process.env.NODE_ENV === "development") {
  checkTimehost = "//10.93.192.225:10010";
} else {
  // checkTimehost = "//10.93.192.225:10010";
  checkTimehost = host;
}

// 视频初始化
const urlSlice = `/api/v1/video/media/init`;
// 文件初始化
const fileuUrlSlice = `/api/v1/file/init`;

const urlStatus = `/api/v1/video/media/showStatus`;

const urlStatusV2 = `/api/v1/video/media/showStatusV2`;

const urlVideoBatch = `/api/v1/video/media/showBatch`;
const urlVideoBatchByKa = `/api/v1/video/media/showBatchByKa`;
const urlVideoBatchByPartKa = `/api/v1/video/media/showBatchByPartKa`;
const urlOriginalVideoInfo = `/api/v1/video/media/showBatchOriginalVideo`;

// 图片关联
const picUrlRelated = "/api/v1/assets/pic/save";
// 视频关联
const videoUrlRelated = "/api/v1/assets/video/save";

//获取时间戳
const getcheckTime = `${checkTimehost}/api/v1/video/media/checkTime`;

interface IurlStatus {
  // uid: string;
  fids: string;
  customerId: string;
  customer_id: string;
  request_source: string;
  isIntranet?: boolean;
  withCredentials?: boolean;
  requestDomain?: string;
  requestPath?: string;
}
interface IurlVideo {
  fids: string;
  customerId: string;
  customer_id: string;
  request_source: string;
  isIntranet?: boolean;
  withCredentials?: boolean;
  part?: string;
  highestQuality?: boolean;
  requestDomain?: string;
  requestPath?: string;
}
// 图片关联参数
interface Ipic {
  pid: string;
  source: number;
}
interface IpicRelated {
  picInfos: Ipic[];
  customerId: string;
}
// 视频关联参数
interface Ivideo {
  fid: string;
  source: number;
  shortUrl: string;
}
interface IvideoRelated {
  videos: Ivideo[];
  customerId: string;
}

class UpstreamApp {
  // uid: string;
  timeout: number;
  isValidPicID: Function;
  getImgURL: Function;
  getImageFormat: Function;
  getImgGeometry: Function;
  getImgLargeFileSize: Function;
  source: any;
  assetsHost: string;
  constructor(params: UpstreamParam) {
    // this.uid = params.uid;
    this.timeout = 10000;
    this.isValidPicID = isValidPicID;
    this.getImgURL = getImgURL;
    this.getImageFormat = getImageFormat;
    this.getImgLargeFileSize = getImgLargeFileSize;
    this.getImgGeometry = getImgGeometry;
    this.source = params.source;
    this.assetsHost = params.assetsHost;
  }
  // 获取切片
  public async getSliceChunk(
    sliceChunkParams: SliceChunkParams,
    isIncrementally = false
  ): Promise<any> {
    observerProgress(sliceChunkParams.progressCallback);
    !isIncrementally &&
      setAutoUpdate(sliceChunkParams?.file?.size / (1024 * 1024));
    let mp4boxfile = MP4Box.createFile();
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    mp4boxfile.onReady = function(info: any) {
      var eventvalue = JSON.stringify(info);
      sendTracking({
        eventid: 1008168, // 申请的事件id
        eventtype: "asset-upload-encodeType", // 事件类型 ，非必须
        eventvalue
      });
    };
    mp4boxfile.onError = function() {};
    file2ArrayBuffer(sliceChunkParams.file)
      .then(res => {
        let arrBuffer = res;
        arrBuffer.fileStart = 0;
        mp4boxfile.appendBuffer(arrBuffer);
        mp4boxfile.flush();
      })
      .catch(err => {
        console.log(err);
      });
    !isIncrementally && updateProgress("fileread");
    let md5 = isIncrementally
      ? await computeMd5Incrementally(sliceChunkParams.file)
      : await computeMd5(sliceChunkParams.file);
    !isIncrementally && updateProgress("md5");
    let response;
    let localmd5 = localStorage.getItem(md5);
    let localinitTime = (localmd5 && JSON.parse(localmd5).initTime) || 0;

    const times = new Date().getTime() - localinitTime < 14400000; //4小时以内

    if (localmd5 && times && JSON.parse(localmd5).data) {
      !isIncrementally && updateProgress("storage");
      return JSON.parse(localStorage.getItem(md5));
    }

    let initTime: any = new Date().getTime();
    let sendParams = {
      name: sliceChunkParams.file.name,
      length: sliceChunkParams.file.size,
      check: md5,
      request_time: initTime,
      source: sliceChunkParams.request_source || this.source,
      customer_id: sliceChunkParams.customerId,
      video_type: sliceChunkParams.video_type
    };
    let domain = sliceChunkParams.requestDomain
      ? sliceChunkParams.requestDomain
      : sliceChunkParams.isIntranet
      ? intranet_host
      : host;
    let requestPath = sliceChunkParams.requestPath || urlSlice;
    // let urls = sliceChunkParams.url
    //   ? sliceChunkParams.url
    //   : `${domain}${urlSlice}`;

    try {
      let url = `${domain}${requestPath}?customer_id=${sliceChunkParams.customerId}&request_source=${sendParams.source}`;
      response = await FileAxios.post(url, sendParams, {
        timeout: this.timeout,
        withCredentials: !sliceChunkParams.isIntranet
      });
      if (response.data.data && response.data.data.upload_id) {
        response.data["md5"] = md5;
        response.data["initTime"] = initTime;
        localStorage.setItem(md5, JSON.stringify(response.data));
      }
      return response.data;
    } catch (e) {}
  }
  // 查询视频状态
  public async getVideoStatus(iurlStatus: IurlStatus): Promise<any> {
    let response: any;
    let source = iurlStatus.request_source || this.source;
    // let domain = iurlStatus.isIntranet ? intranet_host : host;
    let domain = iurlStatus.requestDomain
      ? iurlStatus.requestDomain
      : iurlStatus.isIntranet
      ? intranet_host
      : host;
    let requestPath = iurlStatus.requestPath || urlStatus;
    try {
      response = await FileAxios.get(`${domain}${requestPath}`, {
        params: {
          ...iurlStatus,
          customer_id: iurlStatus.customerId || iurlStatus.customer_id,
          request_source: source
        },
        timeout: this.timeout,
        withCredentials: iurlStatus.hasOwnProperty("withCredentials")
          ? iurlStatus.withCredentials
          : !iurlStatus.isIntranet
      });
      return response;
    } catch (e) {
      console.log(e);
    }
  }
  // 查询视频状态（新）
  public async getVideoStatusV2(iurlStatus: IurlStatus): Promise<any> {
    let response: any;
    let source = iurlStatus.request_source || this.source;
    // let domain = iurlStatus.isIntranet ? intranet_host : host;
    let domain = iurlStatus.requestDomain
      ? iurlStatus.requestDomain
      : iurlStatus.isIntranet
      ? intranet_host
      : host;
    let requestPath = iurlStatus.requestPath || urlStatusV2;
    try {
      response = await FileAxios.get(`${domain}${requestPath}`, {
        params: {
          ...iurlStatus,
          customer_id: iurlStatus.customerId || iurlStatus.customer_id,
          request_source: source
        },
        timeout: this.timeout,
        withCredentials: iurlStatus.hasOwnProperty("withCredentials")
          ? iurlStatus.withCredentials
          : !iurlStatus.isIntranet
      });
      return response;
    } catch (e) {
      console.log(e);
    }
  }
  // 视频封面视频地址
  public async getVideoInfo(iurlVideo: IurlVideo): Promise<any> {
    let response: any;
    let source = iurlVideo.request_source || this.source;
    // let domain = iurlVideo.isIntranet ? intranet_host : host;
    let domain = iurlVideo.requestDomain
      ? iurlVideo.requestDomain
      : iurlVideo.isIntranet
      ? intranet_host
      : host;
    let requestPath = iurlVideo.requestPath
      ? iurlVideo.requestPath
      : iurlVideo.part
      ? urlVideoBatchByPartKa
      : iurlVideo.highestQuality
      ? urlVideoBatchByKa
      : urlVideoBatch;
    try {
      response = await FileAxios.get(`${domain}${requestPath}`, {
        params: {
          ...iurlVideo,
          customer_id: iurlVideo.customerId || iurlVideo.customer_id,
          request_source: source
        },
        timeout: this.timeout,
        withCredentials: iurlVideo.hasOwnProperty("withCredentials")
          ? iurlVideo.withCredentials
          : !iurlVideo.isIntranet
      });
      return response;
    } catch (e) {}
  }

  public async getOriginalVideoInfo(iurlVideo: IurlVideo): Promise<any> {
    let response: any;
    let source = iurlVideo.request_source || this.source;
    let domain = iurlVideo.isIntranet ? intranet_host : host;
    try {
      response = await FileAxios.get(`${domain}${urlOriginalVideoInfo}`, {
        params: {
          ...iurlVideo,
          customer_id: iurlVideo.customerId || iurlVideo.customer_id,
          request_source: source
        },
        timeout: this.timeout,
        withCredentials: iurlVideo.hasOwnProperty("withCredentials")
          ? iurlVideo.withCredentials
          : !iurlVideo.isIntranet
      });
      return response;
    } catch (e) {}
  }

  // 图片关联资产
  public async savePicRelated(ipicRelated: IpicRelated): Promise<any> {
    let response: any;
    const host = this.assetsHost || hostAssets;
    let picUrlRelatedUrl: string =
      host + picUrlRelated + "?customer_id=" + ipicRelated.customerId;
    try {
      response = await FileAxios.post(picUrlRelatedUrl, ipicRelated, {
        timeout: this.timeout,
        withCredentials: true,
        headers: {
          "Content-Type": "application/json;charset=utf-8"
        },
        transformRequest: [
          function(data: any, headers: any) {
            return JSON.stringify(data);
          }
        ]
      });
      return response;
    } catch (e) {}
  }

  // 视频关联资产
  public async saveVideoRelated(ivideoRelated: IvideoRelated): Promise<any> {
    let response: any;
    const host = this.assetsHost || hostAssets;
    let videoUrlRelatedUrl: string =
      host + videoUrlRelated + "?customer_id=" + ivideoRelated.customerId;
    try {
      response = await FileAxios.post(videoUrlRelatedUrl, ivideoRelated, {
        timeout: this.timeout,
        withCredentials: true,
        headers: {
          "Content-Type": "application/json;charset=utf-8"
        },
        transformRequest: [
          function(data: any, headers: any) {
            return JSON.stringify(data);
          }
        ]
      });
      return response;
    } catch (e) {}
  }

  // 文件切片
  public async getFileSliceChunk(
    sliceChunkParams: SliceChunkParams
  ): Promise<any> {
    let md5_file = await computeMd5(sliceChunkParams.file);
    let name = sliceChunkParams.name || sliceChunkParams.file.name;
    let md5_key = `${md5_file}_${encodeURIComponent(name)}${
      sliceChunkParams.file_url ? "_" + sliceChunkParams.file_url : ""
    }`;
    let response;
    let localmd5 = window.localStorage.getItem(md5_key);
    let localinitTime = (localmd5 && JSON.parse(localmd5).initTime) || 0;
    const times = new Date().getTime() - localinitTime < 7200000; //2小时以内
    if (localmd5 && times && JSON.parse(localmd5).data) {
      return JSON.parse(localStorage.getItem(md5_key));
    }
    let sendParams = {
      name,
      length: sliceChunkParams.file.size,
      check: md5_file,
      source: sliceChunkParams.request_source || this.source,
      customer_id: sliceChunkParams.customerId
    };
    let initTime = new Date().getTime();
    let domain = sliceChunkParams.requestDomain
      ? sliceChunkParams.requestDomain
      : sliceChunkParams.isIntranet
      ? intranet_host
      : host;
    let requestPath = sliceChunkParams.requestPath || fileuUrlSlice;
    try {
      response = await FileAxios.post(
        `${domain}${requestPath}?customer_id=${
          sliceChunkParams.customerId
        }&request_source=${sendParams.source}${
          sliceChunkParams.file_url
            ? "&file_url=" + sliceChunkParams.file_url
            : ""
        }`,
        sendParams,
        {
          timeout: this.timeout,
          withCredentials: !sliceChunkParams.isIntranet
        }
      );
      if (response.data.data.uploadID) {
        response.data["md5"] = md5_key;
        response.data["initTime"] = initTime;
        window.localStorage.setItem(md5_key, JSON.stringify(response.data));
        return response.data;
      }
    } catch (e) {}
  }

  public async customUploadFile(params: {
    file: File;
    business_type: number;
    output_file_type: number;
    input_file_type: number;
    customerId: string;
    requestDomain?: string;
    isIntranet?: boolean;
    requestPath?: string;
  }) {
    const formData = new FormData();
    const input_file_type = params.hasOwnProperty("input_file_type")
      ? params["input_file_type"]
      : 0;
    const output_file_type = params.hasOwnProperty("output_file_type")
      ? params["output_file_type"]
      : 0;
    const business_type = params.hasOwnProperty("business_type")
      ? params["business_type"]
      : 0;

    // 添加文件到表单数据
    formData.append("file", params.file, params.file.name);

    formData.append("request_source", "asset");
    formData.append("input_file_type", `${input_file_type}`);
    formData.append("output_file_type", `${output_file_type}`);
    formData.append("business_type", `${business_type}`);
    let response;
    let domain = params.requestDomain
      ? params.requestDomain
      : params.isIntranet
      ? intranet_host
      : host;
    let requestPath = params.requestPath || "/api/v1/file/customUpload";
    try {
      response = await FileAxios.post(
        `${domain}${requestPath}?customer_id=${params.customerId}`,
        formData,
        {
          timeout: this.timeout,
          withCredentials: !params.isIntranet
        }
      );
      return response.data;
    } catch (e) {
      return null;
    }
  }

  public async mergeUploadFile(params: {
    file: File;
    request_source?: string;
    requestDomain?: string;
    isIntranet?: boolean;
    customerId?: string;
  }) {
    const formData = new FormData();
    // 添加文件到表单数据
    formData.append("file", params.file, params.file.name);

    formData.append("request_source", "asset");
    params?.request_source &&
      formData.append("request_source", `${params.request_source}`);

    let response;
    let domain = params.requestDomain
      ? params.requestDomain
      : params.isIntranet
      ? intranet_host
      : host;
    let requestPath = "/api/v1/file/mergeUpload";
    try {
      response = await FileAxios.post(
        `${domain}${requestPath}?customer_id=${params?.customerId || ""}`,
        formData,
        {
          timeout: this.timeout,
          withCredentials: !params.isIntranet
        }
      );
      return response.data;
    } catch (e) {
      return null;
    }
  }
}

export { UpstreamApp, BaseUpload };
